/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * CMA10x Motherboard I/O Address Maps
 *
 * The size of the motherboard i/o address space is 32 Mbytes. Its location
 * in the physical address space of the cpu is given by CFG_MBIO_BASE.
 * This value is determined by the cpu module plugged into the motherboard
 * and is configured elsewhere.
 *
 * All motherboard I/O devices use byte lane 0 (D0-7) only, for their
 * transfers, i.e. only 8 bit, or 1 byte, transfers can take place, so
 * all the registers are only 8 bits wide. All I/O registers within the
 * devices are 8 bytes apart.
 *
 * For big endian addressing, the register will be at byte 7 (the address
 * + 7). For little endian addressing, the register will be at byte 0 (the
 * address + 0). To learn the endianess we must include <asm/byteorder.h>
 */

/* Motherboard I/O Address Maps */
#define CMA_MBIO_ROMLOW_BASE	(CFG_MBIO_BASE+0x0000000)
#define CMA_MBIO_ROMLOW_SIZE	0x800000
#define CMA_MBIO_RTC_BASE	(CFG_MBIO_BASE+0x0800000)
#define CMA_MBIO_RTC_SIZE	0x4000
#define CMA_MBIO_SERPAR_BASE	(CFG_MBIO_BASE+0x0900000)
#define CMA_MBIO_SERIALB_BASE	(CMA_MBIO_SERPAR_BASE+0x00)
#define CMA_MBIO_SERIALA_BASE	(CMA_MBIO_SERPAR_BASE+0x40)
#define CMA_MBIO_PARALLEL_BASE	(CMA_MBIO_SERPAR_BASE+0x80)
#define CMA_MBIO_SERPAR_SIZE	0x90
#define CMA_MBIO_LCD_BASE	(CFG_MBIO_BASE+0x0b00000)
#define CMA_MBIO_LCD_SIZE	0x10
#define CMA_MBIO_DIPSW_BASE	(CFG_MBIO_BASE+0x0c00000)
#define CMA_MBIO_DIPSW_SIZE	0x10
#define CMA_MBIO_SLOT1CFG_BASE	(CFG_MBIO_BASE+0x1100000)
#define CMA_MBIO_SLOT1CFG_SIZE	0x100000
#define CMA_MBIO_SLOT2CFG_BASE	(CFG_MBIO_BASE+0x1200000)
#define CMA_MBIO_SLOT2CFG_SIZE	0x100000
#define CMA_MBIO_SLOT3CFG_BASE	(CFG_MBIO_BASE+0x1300000)
#define CMA_MBIO_SLOT3CFG_SIZE	0x100000
#define CMA_MBIO_ROMHIGH_BASE	(CFG_MBIO_BASE+0x1800000)
#define CMA_MBIO_ROMHIGH_SIZE	0x800000

#include <asm/byteorder.h>

#ifndef __ASSEMBLY__

/* a single CMA10x motherboard i/o register */
typedef
    struct {
#if defined(__LITTLE_ENDIAN)
	unsigned char value;
#endif
	unsigned char filler[7];
#if defined(__BIG_ENDIAN)
	unsigned char value;
#endif
    }
cma_mbio_reg;

extern inline unsigned char
cma_mbio_reg_read(volatile cma_mbio_reg *reg)
{
    unsigned char data = reg->value;
    asm volatile ("eieio" : : : "memory");
    return data;
}

extern inline void
cma_mbio_reg_write(volatile cma_mbio_reg *reg, unsigned char data)
{
    reg->value = data;
    asm volatile ("eieio" : : : "memory");
}

/* MK48T02 RTC registers */
typedef
    struct {
	cma_mbio_reg sram[2040];/* Battery-Backed SRAM */
	cma_mbio_reg clk_ctl;	/* Clock Control Register */
	cma_mbio_reg clk_sec;	/* Clock Seconds Register */
	cma_mbio_reg clk_min;	/* Clock Minutes Register */
	cma_mbio_reg clk_hour;	/* Clock Hour Register */
	cma_mbio_reg clk_day;	/* Clock Day Register */
	cma_mbio_reg clk_date;	/* Clock Date Register */
	cma_mbio_reg clk_month;	/* Clock Month Register */
	cma_mbio_reg clk_year;	/* Clock Year Register */
    }
cma_mbio_rtc;

/* ST16C522 Serial I/O */
typedef
    struct {
	cma_mbio_reg ser_rhr;	/* Receive Holding Register (R, DLAB=0) */
	cma_mbio_reg ser_ier;	/* Interrupt Enable Register (R/W, DLAB=0) */
	cma_mbio_reg ser_isr;	/* Interrupt Status Register (R) */
	cma_mbio_reg ser_lcr;	/* Line Control Register (R/W) */
	cma_mbio_reg ser_mcr;	/* Modem Control Register (R/W) */
	cma_mbio_reg ser_lsr;	/* Line Status Register (R) */
	cma_mbio_reg ser_msr;	/* Modem Status Register (R/W) */
	cma_mbio_reg ser_spr;	/* Scratch Pad Register (R/W) */
    }
cma_mbio_serial;

#define ser_thr	ser_rhr		/* Transmit Holding Register (W, DLAB=0) */
#define ser_brl	ser_rhr		/* Baud Rate Divisor Low Byte (R/W, DLAB=1) */
#define ser_brh	ser_ier		/* Baud Rate Divisor High Byte (R/W, DLAB=1) */
#define ser_fcr	ser_isr		/* FIFO Control Register (W) */
#define ser_nop	ser_lsr		/* No Operation (W) */

/* ST16C522 Parallel I/O */
typedef
    struct {
	cma_mbio_reg par_rdr;	/* Port Read Data Register (R) */
	cma_mbio_reg par_sr;	/* Status Register (R) */
	cma_mbio_reg par_cmd;	/* Command Register (R) */
    }
cma_mbio_parallel;

#define par_wdr	par_rdr		/* Port Write Data Register (W) */
#define par_ios	par_sr		/* I/O Select Register (W) */
#define par_ctl	par_cmd		/* Control Register (W) */

/* HD44780 LCD Display */
typedef
    struct {
	cma_mbio_reg lcd_ccr;	/* Current Character Register (R/W) */
	cma_mbio_reg lcd_bsr;	/* Busy Status Register (R) */
    }
cma_mbio_lcd;

#define lcd_cmd	lcd_bsr		/* Command Register (W) */

/* 8-Position Configuration Switch */
typedef
    struct {
	cma_mbio_reg dip_val;	/* Dip Switch value (R) */
    }
cma_mbio_dipsw;

#endif /* __ASSEMBLY__ */
