/*
 * Simple serial driver for Cogent CMA10x motherboard serial port A
 * for use during boot
 */

#include <ppcboot.h>

#if defined(CONFIG_8xx_CONS_NONE)

#include "cma10x.h"
#include "serial.h"

void
serial_init(ulong cpu_clock, int baudrate)
{
    cma_mbio_serial *sAp = (cma_mbio_serial *)CMA_MBIO_SERIALA_BASE;

    cma_mbio_reg_write(&sAp->ser_ier, 0x00);	/* turn off interrupts */
    serial_setbrg(cpu_clock, baudrate);
    cma_mbio_reg_write(&sAp->ser_lcr, 0x03);	/* 8 data, 1 stop, no parity */
    cma_mbio_reg_write(&sAp->ser_mcr, 0x03);	/* RTS/DTR */
    cma_mbio_reg_write(&sAp->ser_fcr, 0x07);	/* Clear & enable FIFOs */
}

void
serial_setbrg(ulong cpu_clock, int baudrate)
{
    cma_mbio_serial *sAp = (cma_mbio_serial *)CMA_MBIO_SERIALA_BASE;
    unsigned int divisor;
    unsigned char lcr;

    if ((divisor = br_to_div(baudrate)) == 0)
	divisor = DEFDIV;

    lcr = cma_mbio_reg_read(&sAp->ser_lcr);
    cma_mbio_reg_write(&sAp->ser_lcr, lcr|0x80);/* Access baud rate(set DLAB)*/
    cma_mbio_reg_write(&sAp->ser_brl, divisor & 0xff);
    cma_mbio_reg_write(&sAp->ser_brh, (divisor >> 8) & 0xff);
    cma_mbio_reg_write(&sAp->ser_lcr, lcr);	/* unset DLAB */
}

void
serial_putc(const char c)
{
    cma_mbio_serial *sAp = (cma_mbio_serial *)CMA_MBIO_SERIALA_BASE;

    if (c == '\n')
	serial_putc('\r');

    while ((cma_mbio_reg_read(&sAp->ser_lsr) & LSR_THRE) == 0)
	;

    cma_mbio_reg_write(&sAp->ser_thr, c);
}

void
serial_putstr(const char *s)
{
    while (*s != '\0')
	serial_putc(*s++);
}

int
serial_getc(void)
{
    cma_mbio_serial *sAp = (cma_mbio_serial *)CMA_MBIO_SERIALA_BASE;

    while ((cma_mbio_reg_read(&sAp->ser_lsr) & LSR_DR) == 0)
	;

    return ((int)cma_mbio_reg_read(&sAp->ser_rhr) & 0x7f);
}

int
serial_tstc(void)
{
    cma_mbio_serial *sAp = (cma_mbio_serial *)CMA_MBIO_SERIALA_BASE;

    return ((cma_mbio_reg_read(&sAp->ser_lsr) & LSR_DR) != 0);
}

#endif /* CONFIG_8xx_CONS_NONE */

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)

void
kgdb_serial_init(void)
{
    cma_mbio_serial *sBp = (cma_mbio_serial *)CMA_MBIO_SERIALB_BASE;
    unsigned int divisor;

    if ((divisor = br_to_div(CONFIG_KGDB_BAUDRATE)) == 0)
	divisor = DEFDIV;

    cma_mbio_reg_write(&sBp->ser_ier, 0x00);	/* turn off interrupts */
    cma_mbio_reg_write(&sBp->ser_lcr, 0x80);	/* Access baud rate(set DLAB)*/
    cma_mbio_reg_write(&sBp->ser_brl, divisor & 0xff);
    cma_mbio_reg_write(&sBp->ser_brh, (divisor >> 8) & 0xff);
    cma_mbio_reg_write(&sBp->ser_lcr, 0x03);	/* 8 data, 1 stop, no parity */
    cma_mbio_reg_write(&sBp->ser_mcr, 0x03);	/* RTS/DTR */
    cma_mbio_reg_write(&sBp->ser_fcr, 0x07);	/* Clear & enable FIFOs */

    printf("[on cma10x serial port B] ");
}

void
putDebugChar(int c)
{
    cma_mbio_serial *sBp = (cma_mbio_serial *)CMA_MBIO_SERIALB_BASE;

    while ((cma_mbio_reg_read(&sBp->ser_lsr) & LSR_THRE) == 0)
	;

    cma_mbio_reg_write(&sBp->ser_thr, c & 0xff);
}

void
putDebugStr(const char *str)
{
    while (*str != '\0') {
	if (*str == '\n')
	    putDebugChar('\r');
	putDebugChar(*str++);
    }
}

int
getDebugChar(void)
{
    cma_mbio_serial *sBp = (cma_mbio_serial *)CMA_MBIO_SERIALB_BASE;

    while ((cma_mbio_reg_read(&sBp->ser_lsr) & LSR_DR) == 0)
	;

    return ((int)cma_mbio_reg_read(&sBp->ser_rhr) & 0x7f);
}

void
kgdb_interruptible(int yes)
{
    cma_mbio_serial *sBp = (cma_mbio_serial *)CMA_MBIO_SERIALB_BASE;

    if (yes == 1) {
	printf("kgdb: turning serial ints on\n");
	cma_mbio_reg_write(&sBp->ser_ier, 0xf);
    }
    else {
	printf("kgdb: turning serial ints off\n");
	cma_mbio_reg_write(&sBp->ser_ier, 0x0);
    }
}

#endif /* CFG_CMD_KGDB */
