/*
 * Cogent CMA302 I/O Module - Flash Memory and PS/2 keyboard + mouse
 */

/**************** DEFINES for H8542B Keyboard/Mouse Controller ***************/

/*
 * note the auxillary port is used to control the mouse
 */

/* 8542B Commands (Sent to the Command Port) */
#define HT8542_CMD_SET_BYTE	0x60	/* Set the command byte */
#define HT8542_CMD_GET_BYTE	0x20	/* Get the command byte */
#define HT8542_CMD_KBD_OBUFF	0xD2	/* Write to HT8542 Kbd Output Buffer */
#define HT8542_CMD_AUX_OBUFF	0xD3	/* Write to HT8542 Mse Output Buffer */
#define HT8542_CMD_AUX_WRITE	0xD4	/* Write to Mouse Port */
#define HT8542_CMD_AUX_OFF	0xA7	/* Disable Mouse Port */
#define HT8542_CMD_AUX_ON	0xA8	/* Re-Enable Mouse Port */
#define HT8542_CMD_AUX_TEST	0xA9	/* Test for the presence of a Mouse */
#define HT8542_CMD_DIAG		0xAA	/* Start Diagnostics */
#define HT8542_CMD_KBD_TEST	0xAB	/* Test for presence of a keyboard */
#define HT8542_CMD_KBD_OFF	0xAD	/* Disable Kbd Port (use KBD_DAT_ON) */
#define HT8542_CMD_KBD_ON	0xAE	/* Enable Kbd Port (use KBD_DAT_OFF) */

/* HT8542B cmd byte set by KBD_CMD_SET_BYTE and retrieved by KBD_CMD_GET_BYTE */
#define HT8542_CMD_BYTE_TRANS	0x40
#define HT8542_CMD_BYTE_AUX_OFF	0x20	/* 1 = mse port disabled, 0 = enabled */
#define HT8542_CMD_BYTE_KBD_OFF	0x10	/* 1 = kbd port disabled, 0 = enabled */
#define HT8542_CMD_BYTE_OVER	0x08	/* 1 = override keyboard lock */
#define HT8542_CMD_BYTE_RES	0x04	/* reserved */
#define HT8542_CMD_BYTE_AUX_INT	0x02	/* 1 = enable mouse interrupt */
#define HT8542_CMD_BYTE_KBD_INT	0x01	/* 1 = enable keyboard interrupt */

/* Keyboard Commands (Sent to the Data Port) */
#define KBD_CMD_LED		0xED	/* Set Keyboard LEDS with next byte */
#define KBD_CMD_ECHO		0xEE	/* Echo - we get 0xFA, 0xEE back */
#define KBD_CMD_MODE		0xF0	/* set scan code mode with next byte */
#define KBD_CMD_ID		0xF2	/* get keyboard/mouse ID */
#define KBD_CMD_RPT		0xF3	/* Set Repeat Rate and Delay 2nd Byte */
#define KBD_CMD_ON		0xF4	/* Enable keyboard */
#define KBD_CMD_OFF		0xF5	/* Disables Scanning, Resets to Def */
#define KBD_CMD_DEF		0xF6	/* Reverts kbd to default settings */
#define KBD_CMD_RST		0xFF	/* Reset - should get 0xFA, 0xAA back */

/* Set LED second bit defines */
#define KBD_CMD_LED_SCROLL	0x01	/* Set SCROLL LOCK LED on */
#define KBD_CMD_LED_NUM		0x02	/* Set NUM LOCK LED on */
#define KBD_CMD_LED_CAPS	0x04	/* Set CAPS LOCK LED on */

/* Set Mode second byte defines */
#define KBD_CMD_MODE_STAT	0x00	/* get current scan code mode */
#define KBD_CMD_MODE_SCAN1	0x01	/* set mode to scan code 1 */
#define KBD_CMD_MODE_SCAN2	0x02	/* set mode to scan code 2 */
#define KBD_CMD_MODE_SCAN3	0x03	/* set mode to scan code 3 */

/* Keyboard/Mouse ID Codes */
#define KBD_CMD_ID_1ST		0xAB	/* 1st byte is 0xAB, 2nd is actual ID */
#define KBD_CMD_ID_KBD		0x83	/* Keyboard */
#define KBD_CMD_ID_MOUSE	0x00	/* Mouse */

/* Keyboard Data Return Defines */
#define KBD_STAT_OVER		0x00	/* Buffer Overrun */
#define KBD_STAT_DIAG_OK	0x55	/* Internal Self Test OK */
#define KBD_STAT_RST_OK		0xAA	/* Reset Complete */
#define KBD_STAT_ECHO		0xEE	/* Echo Command Return */
#define KBD_STAT_BRK		0xF0	/* Prefix for Break Key Code */
#define KBD_STAT_ACK		0xFA	/* Received after all commands */
#define KBD_STAT_DIAG_FAIL	0xFD	/* Internal Self Test Failed */
#define KBD_STAT_RESEND		0xFE	/* Resend Last Command */

/* HT8542B Status Register Bit Defines */
#define HT8542_STAT_OBF		0x01	/* 1 = output buffer is full */
#define HT8542_STAT_IBF		0x02	/* 1 = input buffer is full */
#define HT8542_STAT_SYS		0x04	/* system flag - unused */
#define HT8542_STAT_CMD		0x08	/* 1 = cmd in input buffer, 0 = data */
#define HT8542_STAT_INH		0x10	/* 1 = Inhibit - unused */
#define HT8542_STAT_TX		0x20	/* 1 = Transmit Timeout has occured */
#define HT8542_STAT_RX		0x40	/* 1 = Receive Timeout has occured */
#define HT8542_STAT_PERR	0x80	/* 1 = Parity Error from Keyboard */

/******************* DEFINES for 28F008S5 FLASH chip *************************/

/* register addresses, valid only following a CHIP_CMD_RD_ID command */
#define CHIP_ADDR_MAN	0x00000	/* manufacturer's id */
#define CHIP_ADDR_DEV	0x00001	/* device id */
#define CHIP_ADDR_CFGM	0x00003	/* master lock configuration */
#define CHIP_ADDR_CFG0	0x00002	/* block lock configuration */
#define CHIP_ADDR_CFG1	0x10002	/* " */
#define CHIP_ADDR_CFG2	0x20002	/* " */
#define CHIP_ADDR_CFG3	0x30002	/* " */
#define CHIP_ADDR_CFG4	0x40002	/* " */
#define CHIP_ADDR_CFG5	0x50002	/* " */
#define CHIP_ADDR_CFG6	0x60002	/* " */
#define CHIP_ADDR_CFG7	0x70002	/* " */
#define CHIP_ADDR_CFG8	0x80002	/* " */
#define CHIP_ADDR_CFG9	0x90002	/* " */
#define CHIP_ADDR_CFG10	0xA0002	/* " */
#define CHIP_ADDR_CFG11	0xB0002	/* " */
#define CHIP_ADDR_CFG12	0xC0002	/* " */
#define CHIP_ADDR_CFG13	0xD0002	/* " */
#define CHIP_ADDR_CFG14	0xE0002	/* " */
#define CHIP_ADDR_CFG15	0xF0002	/* " */

/* Commands */
#define CHIP_CMD_RST	0xFF	/* reset flash */
#define CHIP_CMD_RD_ID	0x90	/* read the id and lock bits */
#define CHIP_CMD_RD_STAT 0x70	/* read the status register */
#define CHIP_CMD_CLR_STAT 0x50	/* clear the staus register */
#define CHIP_CMD_ERASE1	0x20	/* first word for block erase */
#define CHIP_CMD_ERASE2	0xD0	/* second word for block erase */
#define CHIP_CMD_PROG	0x40	/* program word command */
#define CHIP_CMD_LOCK	0x60	/* first word for all lock commands */
#define CHIP_CMD_SET_LOCK_BLK 0x01 /* 2nd word for set block lock bit */
#define CHIP_CMD_SET_LOCK_MSTR 0xF1 /* 2nd word for set master lock bit */
#define CHIP_CMD_CLR_LOCK_BLK 0xD0 /* 2nd word for clear block lock bit */

/* status register bits */
#define CHIP_STAT_DPS	0x02	/* Device Protect Status */
#define CHIP_STAT_PSS	0x04	/* Program Suspend Status */
#define CHIP_STAT_VPPS	0x08	/* VPP Status */
#define CHIP_STAT_PSLBS	0x10	/* Program and Set Lock Bit Status */
#define CHIP_STAT_ECLBS	0x20	/* Erase and Clear Lock Bit Status */
#define CHIP_STAT_ESS	0x40	/* Erase Suspend Status */
#define CHIP_STAT_RDY	0x80	/* Write State Machine Status, 1=rdy */

#define CHIP_STAT_ERR	(CHIP_STAT_VPPS|CHIP_STAT_DPS|\
			    CHIP_STAT_ECLBS|CHIP_STAT_PSLBS)

/* ID and Lock Configuration */
#define CHIP_RD_ID_LOCK	0x01	/* Bit 0 of each byte */
#define CHIP_RD_ID_MAN	0x89	/* Manufacturer code = 0x89 */
#define CHIP_RD_ID_DEV	0xA6	/* Device code = 0xA6, 28F008S5 */

/* dimensions */
#define CHIP_NBLOCKS	16		/* a 28F008S5 consists of 16 blocks */
#define CHIP_BLOCKSZ	(64*1024)	/* of 64Kbyte each */
#define CHIP_SIZE	(CHIP_BLOCKSZ * CHIP_NBLOCKS)

/********************* DEFINES for Cogent CMA302 *****************************/

/*
 * Quoted from the CMA302 manual:
 *
 * Although the CMA302 supports 64-bit reads, all writes must be done with
 * word size only. When programming the CMA302, the FLASH devices appear as 2
 * banks of interleaved, 32-bit wide FLASH. Each 32-bit word consists of four
 * 28F008S5 devices. The first bank is accessed when the word address is even,
 * while the second bank is accessed when the word address is odd. This must
 * be taken into account when programming the desired word. Also, when locking
 * blocks, software must lock both banks. The CMA302 does not directly support
 * byte writing.  Programming and/or erasing individual bytes is done with
 * selective use of the Write Command.  By not placing the Write Command value
 * on a particular byte lane, that byte will not be written with the following
 * Write Data. Also, remember that within a byte lane (i.e. D0-7), there are
 * two 28F008S5 devices, one for each bank or every other word.
 *
 * End quote.
 *
 * Each 28F008S5 is 8Mbit, with 8 bit wide data. i.e. each is 1Mbyte. The
 * chips are arranged on the CMA302 in multiples of two banks, each bank having
 * 4 chips. Each bank must be accessed as a single 32 bit wide device (i.e.
 * aligned on a 32 bit boundary), with each byte lane within the 32 bits (0-3)
 * going to each of the 4 chips and the word address selecting the bank, even
 * being the low bank and odd the high bank. For 64bit reads, both banks are
 * read simultaneously with the second bank on byte lanes 4-7. Each 28F008S5
 * consists of 16 64Kbyte "block"s. Before programming a byte, the block that
 * the byte resides within must be erased. So if you want to program contiguous
 * memory locations, you must erase all 8 chips at the same time. i.e. the
 * flash on the CMA302 can be viewed as a number of 512Kbyte blocks.
 *
 * Note: I am going to treat banks as 8 Mbytes (1Meg of 64bit words), whereas
 * the example code treats them as a pair of interleaved 1 Mbyte x 32bit banks.
 */

typedef unsigned long flash_word_t;	/* 32 or 64 bit unsigned integer */
typedef volatile flash_word_t *flash_addr_t;
typedef unsigned long flash_size_t;	/* want this big - at least 32 bit */

/* layout of banks on cma302 board */
#define BANK_WIDTH	8		/* each bank is 8 chips wide */
#define BANK_SHIFT	3		/* (1 << BANK_SHIFT) == BANK_WIDTH */
#define BANK_NBLOCKS	CHIP_NBLOCKS
#define BANK_BLOCKSZ	(CHIP_BLOCKSZ * BANK_WIDTH)
#define BANK_SIZE	(CHIP_SIZE * BANK_WIDTH)

#define MAX_BANKS	2		/* up to 2 banks (8M each) on CMA302 */

/* align addresses and sizes to bank boundaries */
#define BANK_ALIGN_ADDR(a)	((flash_addr_t)\
				    ((flash_size_t)(a) & ~(BANK_WIDTH - 1)))
#define BANK_ALIGN_SIZE(s)	((flash_size_t)BANK_ALIGN_ADDR((flash_size_t)\
				    (s) + (BANK_WIDTH - 1)))

/* align addresses and sizes to block boundaries */
#define BLOCK_ALIGN_ADDR(a)	((flash_addr_t)\
				    ((flash_size_t)(a) & ~(BANK_BLOCKSZ - 1)))
#define BLOCK_ALIGN_SIZE(s)	((flash_size_t)BLOCK_ALIGN_ADDR((flash_size_t)\
				    (s) + (BANK_BLOCKSZ - 1)))

/* add a byte offset to a flash address */
#define ADD_BYTEOFF_ADDR(a,o)	(flash_addr_t)((flash_size_t)(a) + (o))

/* adjust a bank address with a chip register offset */
#define CHIP_TO_BANK_ADDR(a,o)	ADD_BYTEOFF_ADDR((a), (o) << BANK_SHIFT)

#define BANK_BASE_ADDR(a,b)	ADD_BYTEOFF_ADDR((a), \
				    (flash_size_t)(b) * BANK_SIZE)
#define BANK_NEXT_WORD_ADDR(a)	ADD_BYTEOFF_ADDR((a), BANK_WIDTH)
#define BANK_NEXT_BLOCK_ADDR(a)	ADD_BYTEOFF_ADDR((a), BANK_BLOCKSZ)
#define BANK_NEXT_BANK_ADDR(a)	ADD_BYTEOFF_ADDR((a), BANK_SIZE)

/* make a bank representation for each chip address */

#define BANK_ADDR_MAN(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_MAN)
#define BANK_ADDR_DEV(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_DEV)
#define BANK_ADDR_CFGM(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFGM)
#define BANK_ADDR_CFG0(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG0)
#define BANK_ADDR_CFG1(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG1)
#define BANK_ADDR_CFG2(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG2)
#define BANK_ADDR_CFG3(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG3)
#define BANK_ADDR_CFG4(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG4)
#define BANK_ADDR_CFG5(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG5)
#define BANK_ADDR_CFG6(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG6)
#define BANK_ADDR_CFG7(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG7)
#define BANK_ADDR_CFG8(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG8)
#define BANK_ADDR_CFG9(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG9)
#define BANK_ADDR_CFG10(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG10)
#define BANK_ADDR_CFG11(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG11)
#define BANK_ADDR_CFG12(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG12)
#define BANK_ADDR_CFG13(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG13)
#define BANK_ADDR_CFG14(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG14)
#define BANK_ADDR_CFG15(a)	CHIP_TO_BANK_ADDR((a), CHIP_ADDR_CFG15)

/*
 * replicate a chip cmd/stat/rd value into each byte position within a word
 * so that multiple chips are accessed in a single word i/o operation
 *
 * this must be as wide as the flash_word_t type
 */
#define REPLICATE_TO_WORD(o)	(((unsigned long)(o) << 24) | \
				    ((unsigned long)(o) << 16) | \
				    ((unsigned long)(o) << 8) | \
				    (unsigned long)(o))

/* make a bank representation for each chip cmd/stat/rd value */

/* Commands */
#define BANK_CMD_RST		REPLICATE_TO_WORD(CHIP_CMD_RST)
#define BANK_CMD_RD_ID		REPLICATE_TO_WORD(CHIP_CMD_RD_ID)
#define BANK_CMD_RD_STAT	REPLICATE_TO_WORD(CHIP_CMD_RD_STAT)
#define BANK_CMD_CLR_STAT	REPLICATE_TO_WORD(CHIP_CMD_CLR_STAT)
#define BANK_CMD_ERASE1		REPLICATE_TO_WORD(CHIP_CMD_ERASE1)
#define BANK_CMD_ERASE2		REPLICATE_TO_WORD(CHIP_CMD_ERASE2)
#define BANK_CMD_PROG		REPLICATE_TO_WORD(CHIP_CMD_PROG)
#define BANK_CMD_LOCK		REPLICATE_TO_WORD(CHIP_CMD_LOCK)
#define BANK_CMD_SET_LOCK_BLK	REPLICATE_TO_WORD(CHIP_CMD_SET_LOCK_BLK)
#define BANK_CMD_SET_LOCK_MSTR	REPLICATE_TO_WORD(CHIP_CMD_SET_LOCK_MSTR)
#define BANK_CMD_CLR_LOCK_BLK	REPLICATE_TO_WORD(CHIP_CMD_CLR_LOCK_BLK)

/* status register bits */
#define BANK_STAT_DPS		REPLICATE_TO_WORD(CHIP_STAT_DPS)
#define BANK_STAT_PSS		REPLICATE_TO_WORD(CHIP_STAT_PSS)
#define BANK_STAT_VPPS		REPLICATE_TO_WORD(CHIP_STAT_VPPS)
#define BANK_STAT_PSLBS		REPLICATE_TO_WORD(CHIP_STAT_PSLBS)
#define BANK_STAT_ECLBS		REPLICATE_TO_WORD(CHIP_STAT_ECLBS)
#define BANK_STAT_ESS		REPLICATE_TO_WORD(CHIP_STAT_ESS)
#define BANK_STAT_RDY		REPLICATE_TO_WORD(CHIP_STAT_RDY)

#define BANK_STAT_ERR		REPLICATE_TO_WORD(CHIP_STAT_ERR)

/* ID and Lock Configuration */
#define BANK_RD_ID_LOCK		REPLICATE_TO_WORD(CHIP_RD_ID_LOCK)
#define BANK_RD_ID_MAN		REPLICATE_TO_WORD(CHIP_RD_ID_MAN)
#define BANK_RD_ID_DEV		REPLICATE_TO_WORD(CHIP_RD_ID_DEV)
